#include "../../stdafx.h"

#include "GrayLines.h"
#include "../MyDemo.h"
#include "../../engine/camera/Camera.h"
#include "../../utils/Random.h"


// ------------------------------------------------------------------
//  basic stuff


CGrayLinesEntity::CGrayLinesEntity( const CCamera& camera )
:	mLineRenderer( NULL ),
	mLinePool( MAX_LINES )
{
	mLineRenderer = new CLineRenderer( camera, CResourceId("AlphaParticle.sha"), CResourceId("lineHard.png"), mLinePool );
	assert( mLineRenderer );
	CMyDemo::getInstance().getObjects().addObject( *mLineRenderer );
	mLineRenderer->setVisible(false);
}

CGrayLinesEntity::~CGrayLinesEntity()
{
	CMyDemo::getInstance().getObjects().removeObject( *mLineRenderer );
	delete mLineRenderer;
}


bool CGrayLinesEntity::isActive( CScenario::eScene scene ) const
{
	return scene >= CScenario::WHEEL_QUAD_BLOB_LINES && scene <= CScenario::END;
}

void CGrayLinesEntity::onSceneStart( CScenario::eScene scene )
{
	if( scene == CScenario::WHEEL_QUAD_BLOB_LINES ) {
		mLineRenderer->setVisible(true);
		mLinePool.clear();
	}
}

void CGrayLinesEntity::onSceneEnd( CScenario::eScene scene )
{
	if( scene == CScenario::END ) {
		mLineRenderer->setVisible(false);
	}
}

void CGrayLinesEntity::update()
{
	CScenario::eScene scene = CScenario::getInstance().getCurrScene();
	if( scene != CScenario::FADE && scene != CScenario::END )
		updateSceneLines();
	else
		updateSceneFade();
}

void CGrayLinesEntity::updateSceneLines()
{
	const CScenario& scenario = CScenario::getInstance();
	float timeStart = scenario.getSceneStart( CScenario::WHEEL_QUAD_BLOB_LINES );
	float timeEnd = scenario.getSceneStart( CScenario::FADE );
	float alpha = (scenario.getCurrentTime()-timeStart) / (timeEnd-timeStart);

	const CPerlin& perlin = CMyDemo::getInstance().getPerlin();

	// populate with lines
	if( alpha < 0.3f ) {
		int shouldbe = MAX_LINES*alpha/0.3f;
		while( mLinePool.size() < shouldbe ) {
			SLine& l = mLinePool.add();
			float f = (float)mLinePool.size() / MAX_LINES;

			l.info.endA.x = cosf(2.8f+f*41.0f+cosf(f+1))*50.0f-10.0f;
			l.info.endA.y = sinf(0.2f+f*59.0f+cosf(f+4))*80.0f;
			l.info.endA.z = 100.0f;

			l.info.endB.x = cosf(f*33.0f+sinf(f+2))*50.0f+10.0;
			l.info.endB.y = sinf(2.3f+f*31.0f+cosf(f+9))*80.0f;
			l.info.endB.z = 100.0f;

			l.info.halfWidth = 0.3f + random::randf( 0.2f );
			l.info.color.a = 1.0f;
			l.info.color.r = l.info.color.g = l.info.color.b = 0.1f + random::randf(0.3f);
			l.beforeUpdate();
		}
	}


	int n = mLinePool.size();
	float df = 1.0f / n;
	float f = 0.0f;
	for( int i = 0; i < n; ++i, f += df ) {
		SLine& l = mLinePool.get(i);
		l.beforeUpdate();

		/*
		l.info.endA.x += 13.0f * sinf( f * 7.0f + l.oldInfo.endA.y * 0.13f );
		l.info.endA.y += 11.0f * sinf( f * 11.0f + l.oldInfo.endA.x * 0.23f );

		l.info.endB.x += 4.0f * sinf( f * 13.0f + l.oldInfo.endB.y * 0.13f );
		l.info.endB.y += 2.0f * sinf( f * 10.0f + l.oldInfo.endB.x * 0.23f );
		*/

		/*
		l.info.endA.x += cosf( alpha * 9.2f + sinf( l.oldInfo.endB.x * 0.2f ) ) * 13.0f;
		l.info.endA.y += sinf( l.oldInfo.endA.x * 0.3f + cosf( l.oldInfo.endB.x * 0.3f ) ) * 11.0f;

		l.info.endB.x += sinf( f * 16.7f + cosf( alpha - l.oldInfo.endA.y * 0.13f ) ) * 7.0f;
		l.info.endB.y += cosf( alpha * 11.7f + sinf( l.oldInfo.endB.y ) ) * 11.0f;
		*/

		l.info.endA.x += cosf(l.oldInfo.endB.y * 0.039f + alpha*4.7f+f*2.4f) * (11.0f+alpha*6.0f);
		l.info.endA.y += sinf(l.oldInfo.endB.y * 0.043f + alpha*7.1f-f*3.3f) * (13.0f+alpha*11.0f);
		//l.info.endA.x += cosf( l.oldInfo.endA.y * 0.11f + sinf( f * 5.4f ) * 6.0f ) * (3.0f+alpha*2.0f);
		//l.info.endA.y += sinf( l.oldInfo.endA.x * 0.09f + cosf( f * 2.3f ) * 4.1f ) * (4.0f+alpha*3.0f);

		l.info.endB.x += sinf(l.oldInfo.endA.y * 0.061f + alpha*2.3f-f*6.3f) * (9.0f+alpha*11.0f);
		l.info.endB.y += sinf(l.oldInfo.endA.x * 0.031f + alpha*2.7f+f*4.1f) * (15.0f+alpha*3.0f);
		//l.info.endB.x += sinf( l.oldInfo.endB.x * 0.03f - f * 5.0f ) * 19.0f;
		//l.info.endB.y += cosf( l.oldInfo.endB.y * 0.05f - f * 2.0f ) * 16.0f;

		D3DXMATRIX mat;
		float rotZ = 0.1f - sinf( alpha*8.9f + f * 4.0f ) * 0.06f;
		D3DXMatrixRotationZ( &mat, rotZ );
		D3DXVec3TransformCoord( &l.info.endA, &l.info.endA, &mat );
		D3DXVec3TransformCoord( &l.info.endB, &l.info.endB, &mat );

		l.info.endA.x *= 0.9f;
		l.info.endA.y *= 0.9f;
		l.info.endB.x *= 0.9f;
		l.info.endB.y *= 0.9f;
	}
}

void CGrayLinesEntity::updateSceneFade()
{
	const CScenario& scenario = CScenario::getInstance();

	int n = mLinePool.size();
	float df = 1.0f / n;
	float f = 0.0f;

	// fist line special
	{
		SLine& l = mLinePool.get(0);
		l.beforeUpdate();
		if( scenario.getCurrScene() == CScenario::END && scenario.getCurrSceneAlpha() > 0.75f ) {
			float alpha = 1.0f - (scenario.getCurrSceneAlpha() - 0.75f) * 4.0f;
			alpha = alpha*alpha;
			l.info.endA.x = -alpha*200.0f;
			l.info.endB.x = alpha*200.0f;
		} else {
			if( l.info.endA.x > -200.0f )
				l.info.endA.x -= 4.0f;
			l.info.endA.y *= 0.92f;
			if( l.info.endB.x < 200.0f )
				l.info.endB.x += 4.0f;
			l.info.endB.y *= 0.92f;
			l.info.halfWidth += 0.002f;
		}
	}

	// rest lines
	for( int i = 1; i < n; ++i, f += df ) {
		SLine& l = mLinePool.get(i);
		l.beforeUpdate();

		l.info.endA.x += 6.0f + cosf(l.oldInfo.endB.y * 0.039f + f*2.4f) * 5.0f;
		l.info.endA.y += 1.2f + sinf(l.oldInfo.endB.y * 0.043f - f*3.3f) * 8.0f;

		l.info.endB.x += 6.0f + sinf(l.oldInfo.endA.x * 0.061f - f*6.3f) * 3.0f;
		l.info.endB.y += 1.0f + sinf(l.oldInfo.endA.x * 0.031f + f*4.1f) * 4.0f;
	}
}
