#include "../../stdafx.h"

#include "WhiteLines.h"
#include "../MyDemo.h"
#include "../../engine/camera/Camera.h"
#include "../../utils/Random.h"


// ------------------------------------------------------------------
//  basic stuff


CWhiteLinesEntity::CWhiteLinesEntity( const CCamera& camera )
:	mLineRenderer( NULL ),
	mLinePool( MAX_LINES )
{
	mLineRenderer = new CLineRenderer( camera, CResourceId("AlphaParticle.sha"), CResourceId("lineHard.png"), mLinePool );
	assert( mLineRenderer );
	CMyDemo::getInstance().getObjects().addObject( *mLineRenderer );
}

CWhiteLinesEntity::~CWhiteLinesEntity()
{
	CMyDemo::getInstance().getObjects().removeObject( *mLineRenderer );
	delete mLineRenderer;
}


bool CWhiteLinesEntity::isActive( CScenario::eScene scene ) const
{
	return scene >= CScenario::WHITELINE && scene <= CScenario::LINES_CIRCLE_BLOB || scene >= CScenario::FADE;
}

void CWhiteLinesEntity::onSceneStart( CScenario::eScene scene )
{
	if( scene == CScenario::WHITELINE ) {
		mLineRenderer->setVisible(true);
		mLinePool.clear();

		// 1 line
		SLine& l = mLinePool.add();
		l.info.endA = l.info.endB = D3DXVECTOR3(0,0,100);
		l.info.halfWidth = 0.3f;
		l.info.color = (DWORD)0xFFFFFFFF;
		l.beforeUpdate();
	} else if( scene == CScenario::LINES || scene == CScenario::FADE ) {
		mLineRenderer->setVisible(true);
		mLinePool.clear();

		// many lines
		float dx = 300.0f / MAX_LINES;
		for( int i = 0; i < MAX_LINES; ++i ) {
			SLine& l = mLinePool.add();
			float x1 = -150.0f + i*dx;
			float x2 = x1+dx;
			l.info.endA = D3DXVECTOR3(x1,0,100);
			l.info.endB = D3DXVECTOR3(x2,0,100);
			l.info.halfWidth = 0.3f;
			l.info.color = (DWORD)0xFFFFFFFF;
			l.beforeUpdate();
		}
	}
}

void CWhiteLinesEntity::onSceneEnd( CScenario::eScene scene )
{
	if( scene == CScenario::LINES_CIRCLE_BLOB || scene == CScenario::END ) {
		mLineRenderer->setVisible(false);
	}
}

void CWhiteLinesEntity::update()
{
	switch( CScenario::getInstance().getCurrScene() ) {
	case CScenario::WHITELINE:
		updateSceneWhiteline(); break;
	case CScenario::LINES:
	case CScenario::LINES_BLOB:
	case CScenario::FADE:
	case CScenario::END:
		updateSceneLines(); break;
	case CScenario::LINES_CIRCLE_BLOB:
		updateSceneCircle(); break;
	}
}

void CWhiteLinesEntity::updateSceneWhiteline()
{
	const CScenario& scenario = CScenario::getInstance();
	float alpha = scenario.getCurrSceneAlpha();

	assert( mLinePool.size() >= 1 );

	// 1 line
	SLine& l = mLinePool.get(0);
	l.beforeUpdate();

	float scale;
	float transp;
	if( alpha < 0.5f ) {
		scale = 0.5f;
		transp = 0.2f + random::randf(0.5f);
	} else {
		scale = 0.5f + tanf( alpha * 1.3f ) * 90.0f;
		transp = 1.0f;
	}

	l.info.endA.x = scale;
	l.info.endB.x = -scale;
	l.info.halfWidth = 0.3f;
	l.info.color.a = transp;
}

static void gLinesFunc1( D3DXVECTOR3& res, const D3DXVECTOR3& old, float f, float alpha )
{
	res.x += cosf( old.y * (0.13f + f*0.1f) + sinf( f ) ) * 2.1f;
	res.y += sinf( old.x * 0.11f + cosf( f ) ) * 2.5f;
	res.z = 100.0f + (res.z-100.0f) * 0.9999f;
}

static void gLinesFunc2( D3DXVECTOR3& res, const D3DXVECTOR3& old, float f, float alpha )
{
	float r = 0.8f + alpha;
	res.x += r * sinf( f * 7.0f + old.y * 0.13f );
	res.y += r * sinf( f * 11.0f + old.z * 0.23f );
	res.z += r * cosf( f * 9.0f + old.x * 0.31f );
}

static void gLinesFunc3( D3DXVECTOR3& res, const D3DXVECTOR3& old, float f, float alpha )
{
	D3DXMATRIX mat;
	float a2 = alpha*alpha;
	float rotX = sinf( alpha*7.0f + cosf( (f+0.11f) * 21.0f ) * alpha ) * a2 * 0.2f;
	float rotY = cosf( alpha*5.1f + cosf( f * a2 * 10.0f ) ) * a2 * 0.2f;
	float rotZ = sinf( alpha*8.9f + sinf( f * 7.0f ) * a2 ) * a2 * 0.3f;
	res.z -= 100.0f;
	D3DXMatrixRotationYawPitchRoll( &mat, rotX, rotY, rotZ );
	D3DXVec3TransformCoord( &res, &res, &mat );
	res.z += 100.0f;
}
static void gLinesFunc4( D3DXVECTOR3& res, const D3DXVECTOR3& old, float f, float alpha )
{
	D3DXMATRIX mat;
	float rotZ = -0.4f - sinf( alpha*8.9f + f * 41.0f ) * 0.1f;
	res.z -= 100.0f;
	D3DXMatrixRotationZ( &mat, rotZ );
	D3DXVec3TransformCoord( &res, &res, &mat );
	res.z += 100.0f;
}

void CWhiteLinesEntity::updateSceneLines()
{
	const CScenario& scenario = CScenario::getInstance();
	bool pulse = (scenario.getCurrScene() >= CScenario::FADE);
	float alpha = scenario.getCurrSceneAlpha();
	bool fadeout = (scenario.getCurrScene() == CScenario::END && alpha > 0.75f);
	float fadealpha = 1.0f - (alpha-0.75f)*4.0f;

	if( scenario.getCurrScene() != CScenario::LINES )
		alpha += 1.0f;
	alpha *= 0.5f;

	int n = mLinePool.size();
	float df = 1.0f / n;
	float f1 = 0.0f;
	float f2 = df;
	for( int i = 0; i < n; ++i, f1 += df, f2 += df ) {
		SLine& l = mLinePool.get(i);
		l.beforeUpdate();

		D3DXVECTOR3 res1A = l.info.endA, res1B = l.info.endB;
		gLinesFunc1( res1A, l.oldInfo.endA, f1, alpha );
		gLinesFunc1( res1B, l.oldInfo.endB, f2, alpha );
		float w1 = mPerlin.get1D( alpha ) * 0.5f + 0.5f;

		D3DXVECTOR3 res2A = l.info.endA, res2B = l.info.endB;
		gLinesFunc2( res2A, l.oldInfo.endA, f1, alpha );
		gLinesFunc2( res2B, l.oldInfo.endB, f2, alpha );
		float w2 = 1.0f - w1;

		l.info.endA = res1A * w1 + res2A * w2;
		l.info.endB = res1B * w1 + res2B * w2;

		gLinesFunc3( l.info.endA, l.oldInfo.endA, f1, alpha );
		gLinesFunc3( l.info.endB, l.oldInfo.endB, f2, alpha );

		if( pulse ) {
			float col = 0.3f + sinf( alpha*23.0f + f1 * 9.1f ) * 0.3f;
			l.info.color.r = l.info.color.g = l.info.color.b = col;
		}
		if( fadeout ) {
			l.info.color.r *= fadealpha;
			l.info.color.g *= fadealpha;
			l.info.color.b *= fadealpha;
		}
	}
}

void CWhiteLinesEntity::updateSceneCircle()
{
	const CScenario& scenario = CScenario::getInstance();
	float alpha = scenario.getCurrSceneAlpha();

	int n = mLinePool.size();
	float df = 1.0f / n;
	float f1 = 0.0f;
	float f2 = df;
	float s = 0.8f;
	for( int i = 0; i < n; ++i, f1 += df, f2 += df ) {
		SLine& l = mLinePool.get(i);
		l.beforeUpdate();

		//gLinesFunc2( l.info.endA, l.oldInfo.endA, f1, alpha );
		//gLinesFunc2( l.info.endB, l.oldInfo.endB, f2, alpha );
		gLinesFunc4( l.info.endA, l.oldInfo.endA, f1, alpha );
		gLinesFunc4( l.info.endB, l.oldInfo.endB, f2, alpha );
		float xa = l.info.endA.x;
		float ya = l.info.endA.y;
		float xb = l.info.endB.x;
		float yb = l.info.endB.y;
		float r2a = xa*xa+ya*ya;
		float r2b = xb*xb+yb*yb;
		if( r2a > 30.0f * 30.0f || r2b > 30.0f * 30.0f ) {
			l.info.endA.x *= s;
			l.info.endA.y *= s;
			l.info.endB.x *= s;
			l.info.endB.y *= s;
		}
		l.info.endA.z = (l.info.endA.z-100.0f)*0.9f+100.0f;
		l.info.endB.z = (l.info.endB.z-100.0f)*0.9f+100.0f;
		l.info.color.a = 1.0f - alpha + random::randfs(0.4f);
	}
}
