#include "../stdafx.h"

#include "Mesh.h"
#include "../common/resource/MeshBundle.h"
#include "../common/resource/EffectBundle.h"
#include "../common/resource/TextureBundle.h"
#include "../utils/DXHelpers.h"
#include "../utils/Errors.h"


CMesh::CMesh( CResourceId const& meshId, CResourceId const& effectId, CResourceId const& textureId )
:	mMeshId( meshId ), mEffectId( effectId ), mTextureId( textureId )
{
	D3DXMatrixIdentity( &mWorldMatrix );
	D3DXMatrixIdentity( &mTextureMatrix );
	mColor = 0xFFFFFFFF;
}

CMesh::~CMesh()
{
}

void CMesh::render( CFrameTime const& frameTime )
{
	if( !isVisible() )
		return;
	
	ID3DXMesh* mesh = CMeshBundle::getInstance().getResourceById( mMeshId );
	ID3DXEffect* effect = CEffectBundle::getInstance().getResourceById( mEffectId );

	assert( mesh );
	assert( effect );
	
	applyEffectParameters( *effect );
	
	UINT passes;
	effect->Begin( &passes, 0 );
	for( UINT p = 0; p < passes; ++p ) {
		effect->Pass( p );
		mesh->DrawSubset( 0 );
	}
	effect->End();

	// HACK - else we have memleak
	effect->SetTexture( "texBase", NULL );
}

void CMesh::applyEffectParameters( ID3DXEffect& effect )
{
	IDirect3DTexture8* texture = CTextureBundle::getInstance().getResourceById( mTextureId );
	assert( texture );

	effect.SetTexture( "texBase", texture );
	effect.SetMatrix( "matWorld", &mWorldMatrix );
	effect.SetMatrix( "matTex", &mTextureMatrix );
	effect.SetDword( "dwColor", mColor );
}
