#include "../stdafx.h"

#include "RenderTarget.h"
#include "../utils/Errors.h"


CRenderTarget::CRenderTarget( IDirect3DSurface8& renderTarget, IDirect3DSurface8* zStencilTarget, bool clearBeforeUse, DWORD clearFlag, D3DCOLOR clearColor, float zValue, DWORD stencilValue, bool shouldRelease )
:	mRenderTarget( renderTarget ),
	mZStencilTarget( zStencilTarget ),
	mPreviousRenderTarget( NULL ),
	mPreviousZStencilTarget( NULL ),
	mClearBeforeUse( clearBeforeUse ),
	mClearFlag( clearFlag ),
	mClearColor( clearColor ),
	mZValue( zValue ),
	mStencilValue( stencilValue ),
	mShouldReleaseRT( shouldRelease )
{
	D3DSURFACE_DESC desc;
	mRenderTarget.GetDesc( &desc );
	mWidth = desc.Width;
	mHeight = desc.Height;
	// TBD: render target surface and zstencil surface must be checked for compatibility
}

CRenderTarget* CRenderTarget::createDefaultRenderTarget( IDirect3DDevice8& device, bool clearBeforeUse, DWORD clearFlag, D3DCOLOR clearColor, float zValue, DWORD stencilValue )
{
	// TBD: static list of all received targets, to release later

	HRESULT hres;
	IDirect3DSurface8* targetSurface = NULL;
	IDirect3DSurface8* zstencilSurface = NULL;
	
	hres = device.GetRenderTarget( &targetSurface );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain render target" );
	}
	assert( targetSurface );
	
	hres = device.GetDepthStencilSurface( &zstencilSurface );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain depth-stencil surface" );
	}

	assert( targetSurface );
	return new CRenderTarget( 
		*targetSurface,
		zstencilSurface,
		clearBeforeUse,
		clearFlag,
		clearColor,
		zValue,
		stencilValue );
}

CRenderTarget* CRenderTarget::createRenderTarget( IDirect3DTexture8& texture, IDirect3DSurface8* zStencilTarget, bool clearBeforeUse, DWORD clearFlag, D3DCOLOR clearColor, float zValue, DWORD stencilValue )
{
	// TBD: static list of targets, to release later

	HRESULT hres;
	IDirect3DSurface8* targetSurface = NULL;
	hres = texture.GetSurfaceLevel( 0, &targetSurface );
	if( FAILED( hres ) )
		THROW_DXERROR( hres, "failed to obtain render target surface" );
	assert( targetSurface );

	return new CRenderTarget( 
		*targetSurface,
		zStencilTarget,
		clearBeforeUse,
		clearFlag,
		clearColor,
		zValue,
		stencilValue, true ); // so releases
}

void CRenderTarget::apply()
{
	HRESULT hres;
	IDirect3DDevice8* device = NULL;
	hres = mRenderTarget.GetDevice( &device );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain device" );
	}
	assert( device );
	
	// save render targets
	hres = device->GetRenderTarget( &mPreviousRenderTarget );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain render target" );
	}
	hres = device->GetDepthStencilSurface( &mPreviousZStencilTarget );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain depth-stencil surface" );
	}

	// set new render tartgets
	hres = device->SetRenderTarget(
		&mRenderTarget,
		mZStencilTarget );	
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to set render target" );
	}

	if( mClearBeforeUse ) {
		// clear whole target area
		hres = device->Clear( 0, NULL, mClearFlag, mClearColor, mZValue, mStencilValue );
		if( FAILED( hres ) ) {
			THROW_DXERROR( hres, "failed to clear render target" );
		}
	}

	device->Release();
}

void CRenderTarget::restore()
{
	HRESULT hres;	
	IDirect3DDevice8* device = NULL;
	hres = mRenderTarget.GetDevice( &device );
	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to obtain device" );
	}
	assert( device );
	
	// restore render targets
	hres = device->SetRenderTarget(
		mPreviousRenderTarget,
		mPreviousZStencilTarget );

	device->Release();
	mPreviousRenderTarget->Release();
	mPreviousZStencilTarget->Release();


	if( FAILED( hres ) ) {
		THROW_DXERROR( hres, "failed to restore render target" );
	}
}
