#ifndef __RENDER_TARGET_H
#define __RENDER_TARGET_H


class CRenderTarget {
public:
	CRenderTarget( 
		IDirect3DSurface8& renderTarget,
		IDirect3DSurface8* zStencilTarget,
		bool clearBeforeUse = true,
		DWORD clearFlag = D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, // framebuffer & zbuffer
		D3DCOLOR clearColor = 0xFFFFFFFF, // white
		float zValue = 1.0f, // farthest
		DWORD stencilValue = 0, bool shouldRelease = false );
	~CRenderTarget() { if( mShouldReleaseRT ) mRenderTarget.Release(); }

	static CRenderTarget* createDefaultRenderTarget(
		IDirect3DDevice8& device,
		bool clearBeforeUse = true,
		DWORD clearFlag = D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,	// framebuffer & zbuffer
		D3DCOLOR clearColor = 0xFFFFFFFF,	// white
		float zValue = 1.0f,	// farthest
		DWORD stencilValue = 0 );

	static CRenderTarget* createRenderTarget(
		IDirect3DTexture8& texture,
		IDirect3DSurface8* zStencilTarget,
		bool clearBeforeUse = true,
		DWORD clearFlag = D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,	// framebuffer & zbuffer
		D3DCOLOR clearColor = 0xFFFFFFFF,	// white
		float zValue = 1.0f,	// farthest
		DWORD stencilValue = 0 );

	/*virtual*/void apply();
	/*virtual*/void restore();

	int getWidth() const { return mWidth; }
	int getHeight() const { return mHeight; }

	void	setClearBeforeUse( bool clear ) { mClearBeforeUse = clear; }

private:
	// HACK
	bool				mShouldReleaseRT;
	
	IDirect3DSurface8&	mRenderTarget;
	IDirect3DSurface8*	mZStencilTarget;
	IDirect3DSurface8*	mPreviousRenderTarget;
	IDirect3DSurface8*	mPreviousZStencilTarget;

	bool				mClearBeforeUse;

	DWORD				mClearFlag;
	D3DCOLOR			mClearColor;

	float				mZValue;
	DWORD				mStencilValue;
	
	int					mWidth;
	int					mHeight;
};

#endif
